from __future__ import print_function
import pandas as pd
import requests
import hashlib


class Pwnification(object):

    def __init__(self, plaintext_pass):
        """
        Checks whether your password has been compromised. using Troy Hunt's
        excellent Pwned Passwords repository

        :param plaintext_pass: plaintext entry of the password you want to check pwnage for
        """
        self.pwd = plaintext_pass
        assert len(self.pwd), "Password can't be an empty string. Try again, m9"
        self.hashed_pass = self.hash_pass().upper()
        self.pass_list = self.get_k_anon
        self.is_pass_compromised = self.verify_pwnage(self.pass_list.password)

    def hash_pass(self, hash_algo='sha1'):
        pwd_hash = hashlib.__getattribute__(hash_algo)(self.pwd.encode('utf-8'))
        return pwd_hash.hexdigest()

    @property
    def get_k_anon(self):
        k_anon_endpoint = 'https://api.pwnedpasswords.com/range/'
        pwd_list = requests.get(k_anon_endpoint + self.hashed_pass[:5]).text
        pwd_list = pwd_list.replace('\ufeff', '').upper().split('\r\n')
        pwd_list = [self.hashed_pass[:5] + pwd for pwd in pwd_list]
        pwd_list = [pwd.split(':') for pwd in pwd_list]
        pwd_list = pd.DataFrame(pwd_list, columns=['password', 'frequency'])
        return pwd_list

    def verify_pwnage(self, pwd_list):
        for pwd in pwd_list:
            if pwd == self.hashed_pass:
                return True
        return False


def make_strs(char, incidence, n_freq):
    return char*incidence + char*int(n_freq/2)


def print_stats(freq):
    in_freq = str(freq)
    n_freq = len(in_freq)
    extra_space = ''
    if len(in_freq) % 2:
        n_freq += 1
        extra_space = ' '
    dash_str = make_strs('-', 27, n_freq)
    space_str = make_strs(' ', 31, n_freq)
    print("Your pass has been D0X3D!")
    print("\n")
    print("# " + dash_str + ' STATS ' + dash_str + ' #')
    print("|" + space_str * 2 + " |")
    print("|  Your password appears in the PwnedPasswords database {0} times{1}  |".format(in_freq, extra_space))
    print("|" + space_str * 2 + " |")
    print("# " + dash_str + '-- x --' + dash_str + ' #')
    print("\n")
    print("Get out quick.. Get back in quicker, with an armful of new passes under your arm!")


from __future__ import print_function
from .utils import *
import argparse
import os

__version__ = '0.3.0'


def main():
    """
    Has your pass been compromised? Let's find out!
    """
    parser = argparse.ArgumentParser(add_help=False)
    parser.add_argument('-f', '--forget_pass', action='store_true',
                        help="If this option is enabled, clear the last console line from .bash_history")
    parser.add_argument('password', help="Enter the password you want to check for pwnage")
    parser.add_argument('-h', '--help', action='help',
                        help="""This module queries Troy Hunt's excellent PwnedPasswords repository
                        to see if your password has ever been 1337 h4x0red before.
                        
                        HOW IT WORKS: Using the k-anonimity functionality of PwnedPasswords, hunts_gambit
                        sends the first five characters of your password's hash to PP and retrieves all possible
                        password hashes that begin with those characters. It then processes whether any of these is 
                        _your_ password, and tells you the result in a friendly output message.
                        
                        NOTE: Your full password (neither in plaintext nor hashed) is _never_ sent out to the interwebs.
                        All processing is done locally on your machine""")
    args = parser.parse_args()
    # get plaintext pass
    try:
        plaintext_pass = args.password
    except TypeError:
        print("Gonna need a password to check against there, bud. Run pwnage_checker -h for help")
        return
    # process it against PP
    gambit = Pwnification(plaintext_pass)
    if gambit.is_pass_compromised:
        pwd_list = gambit.pass_list
        freq = pwd_list[pwd_list.password == gambit.hashed_pass].frequency.iloc[0]
        print_stats(freq)
    else:
        print("Yeah you're fine... for now.")
    # clear last line of terminal output
    if args.forget_pass:
        os.system('let "line_num=$HISTCMD-1"; history -d $line_num')


if __name__ == '__main__':
    main()
